import { createStore } from 'vuex'
import { v4 as uuidv4 } from "uuid"
import { instance, fixImageDuration } from "./axios"


export default createStore({
  state: {
    projects: [],
    files: [],
    clips: [],
    errors: [],
    uploads: [],
    project: null,
    user: null,
    preview: {
      file: null,
      clip: null,
      start: 0.0,
      end: 1.0,
      position: 0.0,
      length: 0.0
    },
    scrollToClip: null,
    default_image_length: 30
  },

  mutations: {
    addError(state, message) {
      if (typeof(message) == "object") {
        // Loop through object keys (potentially multiple errors)
        for (const key in message) {
          let error = {'id': uuidv4(), 'message': `${key}: ${message[key]}`}
          state.errors.push(error)
        }
      } else {
        // Single error message
        let error = {'id': uuidv4(), 'message': message}
        state.errors.push(error)
      }
    },
    removeError(state, error) {
      state.errors = state.errors.filter(err => err.id != error.id )
    },
    clearErrors(state) {
      state.errors = []
    },
    setUser(state, user) {
      state.user = user
    },
    setProjects(state, projects) {
      state.projects = projects
    },
    setProject(state, project) {
      state.project = project
    },
    setFiles(state, files) {
      state.files = files
    },
    addFile(state, fileObj) {
      state.files.push(fileObj)
    },
    updateFileThumbnail(state, payload) {
      if (payload.obj.projects.includes(state.project.url)) {
        state.files = [
          ...state.files = state.files.filter(file => file.id != payload.obj.id),
          payload.obj
        ]
      }
    },
    deleteFile(state, file_id) {
      state.files = state.files.filter(file => file.id != file_id)
    },
    addClip(state, clip) {
      state.clips.push(clip)
    },
    setClips(state, clips) {
      state.clips = clips
      if (!clips) {
        // clear latest clip if no clips
        state.scrollToClip = null
      }
    },
    setClip(state, clipObj) {
      state.clips = [
        ...state.clips = state.clips.filter(clip => clip.id != clipObj.id),
        clipObj
      ]
    },
    deleteClip(state, clip_id) {
      state.clips = state.clips.filter(clip => clip.id != clip_id)
    },
    updateClipThumbnail(state, payload) {
      if (state.project.url == payload.obj.project) {
        state.clips = [
          ...state.clips = state.clips.filter(clip => clip.id != payload.obj.id),
          payload.obj
        ]
        if (payload.latest) {
          state.scrollToClip = payload.obj
        }
      }
    },
    setPreview(state, payload) {
      state.preview.start = payload.start
      state.preview.end = payload.end
      if (state.preview.file) {
        state.preview.length = (state.preview.end - state.preview.start) *
            fixImageDuration(state.preview.file.json.duration)
      }
    },
    setPreviewPosition(state, position) {
      state.preview.position = position
    },
    setPreviewFile(state, file) {
      state.preview.file = file
      state.preview.clip = null
      state.preview.start = 0.0
      state.preview.end = 1.0
      if (state.preview.file) {
        state.preview.length = (state.preview.end - state.preview.start) *
            fixImageDuration(state.preview.file.json.duration)
      }
    },
    setPreviewClip(state, clipObj) {
      state.preview.clip = clipObj
      if (clipObj && clipObj.fileObj) {
        state.preview.start = clipObj.start / fixImageDuration(clipObj.fileObj.json.duration)
        state.preview.end = clipObj.end / fixImageDuration(clipObj.fileObj.json.duration)
        state.preview.position = clipObj.start / fixImageDuration(clipObj.fileObj.json.duration)
        if (state.preview.clip.fileObj) {
          state.preview.file = state.preview.clip.fileObj
        }
      } else {
        state.preview.file = null
        state.preview.start = 0.0
        state.preview.end = 1.0
      }

      if (state.preview.file) {
        state.preview.length = (state.preview.end - state.preview.start) *
            fixImageDuration(state.preview.file.json.duration)
      }
    },
    addUpload(state, upload) {
      state.uploads.push(upload)
    },
    updateUpload(state, upload) {
      for (const up of state.uploads) {
        if (up.id == upload.id) {
          up.progress = upload.progress
          break
        }
      }
    },
    removeUpload(state, uploadID) {
      state.uploads = state.uploads.filter(up => up.id != uploadID )
    },
  },
  getters: {
    isAuthenticated: state => !!state.user,
    totalClipDuration: state => {
      return state.clips.map(clip => clip.end - clip.start).reduce((prev, curr) => prev + curr, 0.0)
    }
  },
  actions: {
    async login({commit}, auth) {
      try {
        const response = await instance.get('/users/')
        let user_object = response.data.results[0]
        user_object.auth = auth
        commit('setUser', user_object)
      } catch(err) {
        commit('setUser', null)
        commit('addError', err.response.data)
      }
    },
    async loadProjects({commit}) {
      try {
        const response = await instance.get('/projects/')
        commit('setProjects', response.data.results)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async getProject({commit}, project_id) {
      try {
        return instance.get(`projects/${project_id}/`)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createProject({dispatch, commit}, payload) {
      try {
        await instance.post('/projects/', payload)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async deleteProject({commit, dispatch}, project_id) {
      try {
        await instance.delete(`projects/${project_id}/`)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async loadFiles({commit, dispatch}, project_id) {
      try {
        const response = await instance.get(`projects/${project_id}/files/`)
        let files = response.data.results

        // Generate thumbnail for each file object
        for (let fileObj of files) {
          // fix long duration on images
          fileObj.json.duration = fixImageDuration(fileObj.json.duration)
          let payload = { obj: fileObj, frame: 1 }
          dispatch('attachThumbnail', payload)
        }
        commit('setFiles', files)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async loadClips({commit, dispatch}, project_id) {
      try {
        const response = await instance.get(`projects/${project_id}/clips/`)
        let clips = response.data.results

        // Attach file obj to clip
        for (let clipObj of clips) {
          const file_response = await instance.get(clipObj.file)
          clipObj.fileObj = file_response.data
          commit('addClip', clipObj)

          let fps = clipObj.json.reader.fps.num / clipObj.json.reader.fps.den
          let thumbnail_payload = { obj: clipObj, frame: clipObj.start * fps, latest: false }
          dispatch('attachThumbnail', thumbnail_payload)
        }
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createClip({dispatch, commit}, payload) {
      try {
        const response = await instance.post(`${payload.project_url}clips/`, payload.data)
        let clipObj = response.data
        const file_response = await instance.get(clipObj.file)
        clipObj.fileObj = file_response.data
        commit('addClip', clipObj)

        let fps = clipObj.json.reader.fps.num / clipObj.json.reader.fps.den
        let thumbnail_payload = { obj: clipObj, frame: clipObj.start * fps, latest: true }
        dispatch('attachThumbnail', thumbnail_payload)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async editClip({dispatch, commit}, clipObj) {
      try {
        await instance.patch(`${clipObj.url}`, clipObj)
        commit('setClip', clipObj)
        let fps = clipObj.json.reader.fps.num / clipObj.json.reader.fps.den
        let thumbnail_payload = { obj: clipObj, frame: clipObj.start * fps, latest: true }
        dispatch('attachThumbnail', thumbnail_payload)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async deleteClip({commit}, clip_id) {
      try {
        await instance.delete(`clips/${clip_id}/`)
        commit('deleteClip', clip_id)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async deleteFile({commit}, file_id) {
      try {
        await instance.delete(`files/${file_id}/`)
        commit('deleteFile', file_id)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createFile({commit, dispatch}, payload) {
      let uploadID = uuidv4()
      let uploadObject = { 'id': uploadID, progress: 0 }
      commit('addUpload', uploadObject)

      const config = {
        onUploadProgress: progressEvent => {
          let progress = (progressEvent.loaded / progressEvent.total) * 100;
          let uploadObject = { 'id': uploadID, progress }
          commit('updateUpload', uploadObject)
        }
      }

      try {
        let response = await instance.post(`${payload.project_url}files/`, payload.data, config)
        let fileObj = response.data
        // fix long duration on images
        fileObj.json.duration = fixImageDuration(fileObj.json.duration)
        commit('removeUpload', uploadID)

        let thumbnail_payload = { obj: fileObj, frame: 1 }
        dispatch('attachThumbnail', thumbnail_payload)
      } catch(err) {
        commit('removeUpload', uploadID)
        commit('addError', err.response.data)
      }
    },
    async attachThumbnail({commit}, payload) {
      let fileObj = null
      let objectType = null
      if (payload.obj.file) {
        // Clip passed in
        fileObj = payload.obj.fileObj
        objectType = 'clip'
      } else {
        // File passed in
        fileObj = payload.obj
        objectType = 'file'
      }

      let data = {
        "frame_number": payload.frame,
        "width": 480,
        "height": 270,
        "image_format": "JPEG",
        "image_quality": 100
      }
      try {
        const response = await instance.post(`${fileObj.url}thumbnail/`, data, { responseType: 'arraybuffer' })
        let blob = new Blob(
            [response.data],
            { type: response.headers['content-type'] }
        )
        // Append thumbnail attribute to each file object
        let thumbnailUrl = URL.createObjectURL(blob)
        payload.obj.thumbnail = thumbnailUrl
        if (objectType == 'file') {
          commit('updateFileThumbnail', payload)
        } else {
          commit('updateClipThumbnail', payload)
        }

      } catch(err) {
        commit('addError', err.response.data)
      }
    }
  },
  modules: {
  }
})
